@@ -1,6 +1,7 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
-# Author: "Chris Ward <cward@redhat.com>
+# Author: "Chris Ward" <cward@redhat.com>
 
 '''
 metrique.argparsers
@@ -26,6 +27,8 @@ See `python cube --help` for more details of the config
 options that can be overriden at runtime.
 '''
 
+from __future__ import unicode_literals
+
 import argparse
 import simplejson as json
 
