@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
 # Author: "Chris Ward" <cward@redhat.com>
 
@@ -27,14 +28,14 @@ up by persisting those objects to an external `metriqued` host.
 And finishing with some querying and simple charting of the data.
 
     >>> from metrique import pyclient
-    >>> g = pyclient(cube="osinfo_rpm"")
+    >>> g = pyclient(cube="osinfo_rpm")
     >>> g.get_objects()  # get information about all installed RPMs
-    >>> 'Total RPMs: %s' % len(objects)
-    >>> 'Example Object:', objects[0]
+    >>> 'Total RPMs: %s' % len(g.objects)
+    >>> 'Example Object:', g.objects[0]
         {'_oid': 'dhcp129-66.brq.redhat.com__libreoffice-ure-4.1.4.2[...]',
          '_start': 1390619596.0,
          'arch': 'x86_64',
-         'host': 'dhcp129-66.brq.redhat.com',
+         'host': 'bla.host.com',
          'license': '(MPLv1.1 or LGPLv3+) and LGPLv3 and LGPLv2+ and[...]',
          'name': 'libreoffice-ure',
          'nvra': 'libreoffice-ure-4.1.4.2-2.fc20.x86_64',
@@ -47,12 +48,7 @@ And finishing with some querying and simple charting of the data.
          'summary': 'UNO Runtime Environment',
          'version': '4.1.4.2'
     }
-    >>> # connect to metriqued host to save the objects
-    >>> config_file = '~/.metrique/etc/metrique.json'  # default location
-    >>> m = pyclient(config_file=config_file)
-    >>> osinfo_rpm = m.get_cube('osinfo_rpm')
-    >>> osinfo_rpm.cube_register()  # (run once) register the new cube with the
-    >>> ids = osinfo_rpm.extract()  # alias for get_objects + save_objects
+    >>> _ids = osinfo_rpm.get_objects(flush=True)  # persist to mongodb
     >>> df = osinfo_rpm.find(fields='license')
     >>> threshold = 5
     >>> license_k = df.groupby('license').apply(len)
@@ -69,6 +65,8 @@ And finishing with some querying and simple charting of the data.
     valid date format: '%Y-%m-%d %H:%M:%S,%f', '%Y-%m-%d %H:%M:%S', '%Y-%m-%d'
 '''
 
+from __future__ import unicode_literals
+
 from collections import Mapping, MutableMapping
 from copy import copy
 import glob
@@ -78,11 +76,16 @@ import pandas as pd
 import re
 import urllib
 
-from metrique.config import Config
 from metrique.utils import get_cube, utcnow, jsonhash, dt2ts
+from metrique.jsonconf import JSONConf
 
 logger = logging.getLogger(__name__)
 
+# if HOME environment variable is set, use that
+# useful when running 'as user' with root (supervisord)
+ETC_DIR = os.environ.get('METRIQUE_ETC')
+DEFAULT_CONFIG = os.path.join(ETC_DIR, 'metrique.json')
+
 FIELDS_RE = re.compile('[\W]+')
 SPACE_RE = re.compile('\s+')
 UNDA_RE = re.compile('_')
@@ -93,14 +96,13 @@ TIMESTAMP_OBJ_KEYS = set(['_end', '_start'])
 
 
 class MetriqueObject(Mapping):
-    def __init__(self, _oid, strict=False, touch=False, **kwargs):
+    def __init__(self, _oid, strict=False, **kwargs):
         self._strict = strict
-        self._touch = touch
         self.store = {
             '_oid': _oid,
             '_id': None,
             '_hash': None,
-            '_start': None,
+            '_start': utcnow(),
             '_end': None,
         }
         self._update(kwargs)
@@ -113,9 +115,9 @@ class MetriqueObject(Mapping):
                 if self._strict:
                     raise KeyError("%s is immutable" % key)
                 else:
-                    logger.debug("%s is immutable; not setting" % key)
+                    #logger.debug("%s is immutable; not setting" % key)
                     return
-            if key in TIMESTAMP_OBJ_KEYS:
+            if key in TIMESTAMP_OBJ_KEYS and value is not None:
                 # ensure normalized timestamp
                 value = dt2ts(value)
             if value == '' or value != value:
@@ -156,12 +158,12 @@ class MetriqueObject(Mapping):
         else:
             # if the object is 'current value' without _end,
             # use just str of _oid
-            _id = str(_oid)
+            _id = unicode(_oid)
         return _id
 
     def _gen_hash(self):
         o = copy(self.store)
-        keys = set(o.keys())
+        keys = set(o.iterkeys())
         [o.pop(k) for k in HASH_EXCLUDE_KEYS if k in keys]
         return jsonhash(o)
 
@@ -178,8 +180,6 @@ class MetriqueObject(Mapping):
         # object is 'current value' continuous
         # so update _start to reflect the time when
         # object's current state was (re)set
-        if not self.store.get('_start') or self._touch:
-            self.store['_start'] = utcnow()
         self._validate_start_end()
         # _id depends on _hash
         # so first, _hash, then _id
@@ -194,6 +194,36 @@ class MetriqueObject(Mapping):
 
 
 class MetriqueContainer(MutableMapping):
+    '''
+    Essentially, cubes are data made from a an object id indexed (_oid)
+    dictionary (keys) or dictionary "objects" (values)
+
+    All objects are expected to contain a `_oid` key value property. This
+    property should be unique per individual "object" defined.
+
+    For example, if we are storing logs, we might consider each log line a
+    separate "object" since those log lines should never change in the future
+    and give each a unique `_oid`. Or if we are storing data about
+    'meta objects' of some sort, say 'github repo issues' for example, we
+    might have objects with _oids of
+    `%(username)s_%(reponame)s_%(issuenumber)s`.
+
+    Optionally, objects can contain the following additional meta-properties:
+        * _start - datetime when the object state was set
+        * _end - datetime when the object state changed to a new state
+
+    Field names (object dict keys) must consist of alphanumeric and underscore
+    characters only.
+
+    Field names are partially normalized automatically:
+        * non-alphanumeric characters are removed
+        * spaces converted to underscores
+        * letters are lowercased
+
+    Property values are normalized to some extent automatically as well:
+        * empty strings -> None
+
+    '''
     def __init__(self, objects=None):
         self.store = {}
         if objects is None:
@@ -207,7 +237,7 @@ class MetriqueContainer(MutableMapping):
             self.store = objects
         else:
             raise ValueError(
-                "objects must be a list, tuple, dict or pandas.DataFrame")
+                "objs must be None, a list, tuple, dict or MetriqueContainer")
 
     def __getitem__(self, key):
         return dict(self.store[key])
@@ -253,39 +283,60 @@ class MetriqueContainer(MutableMapping):
         return pd.DataFrame(tuple(self.store))
 
 
+class MetriqueConfig(JSONConf):
+    '''
+    Client default config class. All metrique clients should subclass
+    from their config objects from this class to ensure defaults
+    values are available.
+
+    This configuration class defines the following overrideable defaults.
+
+    To customize local client configuration, add/update
+    `~/.metrique/etc/metrique.json` (default).
+
+    :param batch_size: The number of objs save at a time (5000)
+    :param cube_pkgs: list of package names where to search for cubes ('cubes')
+    :param cube_paths: Additional paths to search for client cubes (None)
+    :param debug: turn on debug mode logging (level: INFO)
+    :param logfile: filename for logs ('metrique.log')
+    :param log2file: boolean - log output to file? (False)
+    :param logstout: boolean - log output to stdout? (True)
+    :param max_workers: number of workers for threaded operations (#cpus)
+    :param password: the password to connect to metriqued with (None)
+    :param sql_retries: number of attempts to run sql queries before excepting
+    :param sql_batch_size: number of objects to sql query for at a time (1000)
+    :param username: the username to connect to metriqued with ($USERNAME)
+    '''
+    default_config = DEFAULT_CONFIG
+    default_config_dir = ETC_DIR
+
+    def __init__(self, config_file=None, **kwargs):
+        config = {
+            'batch_size': 1000,
+            'cube_pkgs': ['cubes'],
+            'cube_paths': [],
+            'debug': True,
+            'logfile': 'metrique.log',
+            'log2file': True,
+            'logstdout': False,
+            'max_workers': 4,
+            'sql_retries': 1,
+            'sql_batch_size': 500
+        }
+        # apply defaults
+        self.config.update(config)
+        # update the config with the args from the config_file
+        super(MetriqueConfig, self).__init__(config_file=config_file)
+        # anything passed in explicitly gets precedence
+        self.config.update(kwargs)
+
+
 class BaseClient(object):
     '''
     Low level client API which provides baseline functionality, including
     methods for loading data from csv and json, loading metrique client
     cubes, config file loading and logging setup.
 
-    Essentially, cubes are data made from a list of dicts.
-
-    All objects are expected to contain a `_oid` key value property. This
-    property should be unique per individual "object" defined.
-
-    For example, if we are storing logs, we might consider each log line a
-    separate "object" since those log lines should never change in the future
-    and give each a unique `_oid`. Or if we are storing data about
-    'meta objects' of some sort, say 'github repo issues' for example, we
-    might have objects with _oids of
-    `%(username)s_%(reponame)s_%(issuenumber)s`.
-
-    Optionally, objects can contain the following additional meta-properties:
-        * _start - datetime when the object state was set
-        * _end - datetime when the object state changed to a new state
-
-    Field names (object dict keys) must consist of alphanumeric and underscore
-    characters only.
-
-    Field names are partially normalized automatically:
-        * non-alphanumeric characters are removed
-        * spaces converted to underscores
-        * letters are lowercased
-
-    Property values are normalized to some extent automatically as well:
-        * empty strings -> None
-
     Additionally, some common operation methods are provided for
     operations such as loading a HTTP uri and determining currently
     configured username.
@@ -334,7 +385,7 @@ class BaseClient(object):
         if self.config is None:
             self.config = {}
 
-        self._config_file = config_file or Config.default_config
+        self._config_file = config_file or MetriqueConfig.default_config
 
         # all defaults are loaded, unless specified in
         # metrique_config.json
@@ -539,9 +590,9 @@ class BaseClient(object):
         :param config: config file or :class:`metrique.jsonconf.JSONConf`
         :param kwargs: additional config key:value pairs to store
         '''
-        if type(config) is type(Config):
+        if type(config) is type(MetriqueConfig):
             self._config_file = config.config_file
         else:
             self._config_file = config or self._config_file
-            self.config = Config(config_file=self._config_file)
+            self.config = MetriqueConfig(config_file=self._config_file)
         self.config.update(kwargs)
