@@ -1,6 +1,7 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
-# Author: "Chris Ward <cward@redhat.com>
+# Author: "Chris Ward" <cward@redhat.com>
 
 '''
 metrique.cubes.gitdata.repo
@@ -12,6 +13,8 @@ for exctacting data from a git repository.
 .. note:: This cube requires python 2.7+
 '''
 
+from __future__ import unicode_literals
+
 import logging
 logger = logging.getLogger(__name__)
 
