@@ -1,6 +1,7 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
-# Author: "Chris Ward <cward@redhat.com>
+# Author: "Chris Ward" <cward@redhat.com>
 
 '''
 metrique.cubes.sqldata.generic
@@ -10,6 +11,8 @@ This module contains the cube methods for extracting
 data from generic SQL data sources.
 '''
 
+from __future__ import unicode_literals
+
 import logging
 logger = logging.getLogger(__name__)
 
@@ -25,10 +28,10 @@ except ImportError:
     HAS_JOBLIB = False
     logger.warn("joblib package not found!")
 
+import os
 import pytz
 import re
 import simplejson as json
-import time
 import traceback
 
 from metrique import pyclient
@@ -75,6 +78,7 @@ class Generic(pyclient):
         self.retry_on_error = None
 
         self._setup_inconsistency_log()
+        self._auto_reconnect_attempted = False
 
     def activity_get(self, ids=None):
         '''
@@ -99,7 +103,6 @@ class Generic(pyclient):
         activities = self.activity_get(oids)
         objects = self.objects.values()
         for doc in objects:
-            doc = doc.as_dict(pop=['_id', '_hash'])
             _oid = doc['_oid']
             acts = activities.setdefault(_oid, [])
             objs = self._activity_import_doc(doc, acts)
@@ -191,9 +194,9 @@ class Generic(pyclient):
 
     def _build_rows(self, rows):
         logger.debug('Building dict_rows from sql_rows(%i)' % len(rows))
-        _rows = {}
+        _rows = defaultdict(list)
         for row in rows:
-            _rows.setdefault(row['_oid'], []).append(row)
+            _rows[row['_oid']].append(row)
         return _rows
 
     def _build_objects(self, rows):
@@ -209,7 +212,6 @@ class Generic(pyclient):
                 objects.append(obj)
             else:
                 objects.append(col_rows[0])
-        logger.debug('... done')
         return objects
 
     def _convert(self, value, field):
@@ -265,9 +267,9 @@ class Generic(pyclient):
             # get a list of all known object ids
             table = self.get_property('table')
             db = self.get_property('db')
-            _id = self.get_property('column')
-            sql = 'SELECT DISTINCT %s.%s FROM %s.%s' % (table, _id, db, table)
-            rows = self.sql_proxy.fetchall(sql)
+            _oid = self.get_property('_oid')
+            sql = 'SELECT DISTINCT %s.%s FROM %s.%s' % (table, _oid, db, table)
+            rows = self.fetchall(sql)
             oids = self._extract_row_ids(rows)
         else:
             if self.get_property('delta_new_ids', default=True):
@@ -280,6 +282,56 @@ class Generic(pyclient):
                     oids.extend(self.get_changed_oids(mtime))
         return sorted(set(oids))
 
+    def fetchall(self, sql, cached=True):
+        '''
+        Shortcut for getting a cursor, cleaning the sql a bit,
+        adding the LIMIT clause, executing the sql, fetching
+        all the results
+
+        If certain failures occur, this method will authomatically
+        attempt to reconnect and rerun.
+
+        :param sql: sql string to execute
+        :param cached: flag for using a cached proxy or not
+        '''
+        logger.debug('Fetching rows...')
+        proxy = self.get_sql_proxy(cached=cached)
+        k = proxy.cursor()
+        sql = re.sub('\s+', ' ', sql).strip().encode('utf-8')
+        logger.debug('SQL:\n %s' % sql.decode('utf-8'))
+        rows = None
+        try:
+            k.execute(sql)
+            rows = k.fetchall()
+        except Exception as e:
+            if re.search('Transaction is not active', str(e)):
+                if not self._auto_reconnect_attempted:
+                    logger.warn('Transaction failure; reconnecting')
+                    self.fetchall(sql, cached=False)
+            logger.error('%s\n%s\n%s' % ('*' * 100, e, sql))
+            raise
+        else:
+            if self._auto_reconnect_attempted:
+                # in the case we've attempted to reconnect and
+                # the transaction succeeded, reset this flag
+                self._auto_reconnect_attempted = False
+        finally:
+            k.close()
+            del k
+        logger.debug('... fetched (%i)' % len(rows))
+        return rows
+
+    def get_sql_proxy(self, **kwargs):
+        '''
+        Database specific drivers must implemented this method.
+
+        It is expected that by calling this method, the instance
+        will set ._sql_proxy with a auhenticated connection, which is
+        also returned to the caller.
+        '''
+        raise NotImplementedError(
+            "Driver has not provided a get_sql_proxy method!")
+
     def get_full_history(self, force=None, last_update=None,
                          parse_timestamp=None, flush=False, autosnap=False):
         '''
@@ -321,19 +373,13 @@ class Generic(pyclient):
             [self.objects.add(obj) for obj in result]
             return self
 
-    def _fetchall(self, sql, field_order):
-        rows = self.sql_proxy.fetchall(sql)
-        if not rows:
-            return []
-
-        # FIXME: This unicode stuff is fragile and likely to fail
-
+    def _unwrap_aggregated(self, rows):
         # unwrap aggregated values
+        # FIXME: This unicode stuff is fragile and likely to fail
+        encoding = self.get_property('encoding', default=DEFAULT_ENCODING)
         for k, row in enumerate(rows):
             _row = []
             for column in row:
-                encoding = self.get_property('encoding',
-                                             default=DEFAULT_ENCODING)
                 if type(column) is buffer:
                     # unwrap/convert the aggregated string 'buffer'
                     # objects to string
@@ -348,14 +394,15 @@ class Generic(pyclient):
                         column = unicode(column.decode(encoding))
                 _row.append(column)
             rows[k] = _row
+        return rows
 
+    def _fetchall(self, sql, field_order):
+        rows = self.fetchall(sql)
+        if not rows:
+            return []
         logger.debug('Preparing row data...')
-        k = len(rows)
-        t0 = time.time()
+        rows = self._unwrap_aggregated(rows)
         objects = [self._prep_object(row, field_order) for row in rows]
-        t1 = time.time()
-        logger.debug('... Rows prepared %i docs (%i/sec)' % (
-            k, float(k) / (t1 - t0)))
         return objects
 
     def _fetch_mtime(self, last_update, parse_timestamp):
@@ -393,11 +440,15 @@ class Generic(pyclient):
         '''
         Dictionary of field_id: field_name, as defined in self.fields property
         '''
-        fieldmap = defaultdict(str)
-        for field in self.fields:
-            field_id = self.get_property('what', field)
-            if field_id is not None:
-                fieldmap[field_id] = field
+        if hasattr(self, '_sql_fieldmap') and self._sql_fieldmap:
+            fieldmap = self._sql_fieldmap
+        else:
+            fieldmap = defaultdict(str)
+            for field in self.fields:
+                field_id = self.get_property('what', field)
+                if field_id is not None:
+                    fieldmap[field_id] = field
+            self._sql_fieldmap = fieldmap
         return fieldmap
 
     def _gen_sql(self, id_delta, field_order):
@@ -424,20 +475,18 @@ class Generic(pyclient):
 
         sql = 'SELECT %s %s %s %s' % (
             selects, froms, left_joins, where)
-
-        sql += self._sql_sort(table)
+        sql = self._sql_sort(sql, table)
         sql = self._sql_distinct(sql)
-        logger.debug('... done')
         return sql
 
     def _get_id_delta_sql(self, table, id_delta):
         '''
         '''
-        _id = self.get_property('column')
+        _oid = self.get_property('_oid')
         if id_delta:
             id_delta = sorted(set(id_delta))
             id_delta = ','.join(map(str, id_delta))
-            return ["(%s.%s IN (%s))" % (table, _id, id_delta)]
+            return ["(%s.%s IN (%s))" % (table, _oid, id_delta)]
         else:
             return []
 
@@ -463,21 +512,11 @@ class Generic(pyclient):
             return []
         if isinstance(mtime_columns, basestring):
             mtime_columns = [mtime_columns]
-
-        filters = []
+        where = []
         for _column in mtime_columns:
             _sql = "%s > %s" % (_column, mtime)
-            filters.append(_sql)
-
-        db = self.get_property('db')
-        table = self.get_property('table')
-        _id = self.get_property('column')
-
-        sql = """SELECT DISTINCT %s.%s FROM %s.%s
-            WHERE %s""" % (table, _id, db, table,
-                           ' OR '.join(filters))
-        rows = self.sql_proxy.fetchall(sql) or []
-        return [x[0] for x in rows]
+            where.append(_sql)
+        return self.sql_get_oids(where)
 
     def get_metrics(self, names=None):
         '''
@@ -500,7 +539,7 @@ class Generic(pyclient):
             sql = definition['sql']
             fields = definition['fields']
             _oid = definition['_oid']
-            rows = self.sql_proxy.fetchall(sql)
+            rows = self.fetchall(sql)
             for row in rows:
                 d = copy(obj)
 
@@ -535,11 +574,11 @@ class Generic(pyclient):
         field_order = tuple(self.fields)
 
         batch_size = self.config.sql_batch_size
-        max_workers = self.config.max_workers
         kwargs = self.config
         kwargs.pop('cube', None)  # ends up in config; ignore it
-        runner = Parallel(n_jobs=max_workers)
         if HAS_JOBLIB:
+            max_workers = self.config.max_workers
+            runner = Parallel(n_jobs=max_workers)
             func = delayed(get_objects)
             result = runner(func(
                 cube=self._cube, oids=batch, flush=flush,
@@ -573,7 +612,7 @@ class Generic(pyclient):
         '''
         table = self.get_property('table')
         db = self.get_property('db')
-        _id = self.get_property('column')
+        _oid = self.get_property('_oid')
         last_id = self.get_last_field('_oid')
         if last_id:
             # if we delta_new_ids is on, but there is no 'last_id',
@@ -583,12 +622,12 @@ class Generic(pyclient):
             except (TypeError, ValueError):
                 pass
             if type(last_id) in [int, float]:
-                where = "%s.%s > %s" % (table, _id, last_id)
+                where = "%s.%s > %s" % (table, _oid, last_id)
             else:
-                where = "%s.%s > '%s'" % (table, _id, last_id)
+                where = "%s.%s > '%s'" % (table, _oid, last_id)
             sql = 'SELECT DISTINCT %s.%s FROM %s.%s WHERE %s' % (
-                table, _id, db, table, where)
-            rows = self.sql_proxy.fetchall(sql)
+                table, _oid, db, table, where)
+            rows = self.fetchall(sql)
             ids = self._extract_row_ids(rows)
         else:
             ids = []
@@ -598,7 +637,7 @@ class Generic(pyclient):
         '''
         '''
         clauses = []
-        for f in self.fields.keys():
+        for f in self.fields.iterkeys():
             try:
                 _clause = self.fields[f]['sql'].get(clause)
             except KeyError:
@@ -619,9 +658,9 @@ class Generic(pyclient):
 
     def _get_sql_selects(self, field_order):
         table = self.get_property('table')
-        _id = self.get_property('column')
+        _oid = self.get_property('_oid')
 
-        base_select = '%s.%s' % (table, _id)
+        base_select = '%s.%s' % (table, _oid)
         selects = [base_select]
         for f in field_order:
             try:
@@ -737,7 +776,7 @@ class Generic(pyclient):
             column = self._normalize_container(column, field)
             column = self._convert(column, field)
             column = self._type(column, field)
-            obj.update({field: column})
+            obj[field] = column
         return obj
 
     def __row_iter(self, rows):
@@ -751,6 +790,8 @@ class Generic(pyclient):
         _logfile = self.config.logfile.split('.log')[0]
         basename = _logfile + '.inconsistencies'
         logfile = basename + '.log'
+        logdir = os.environ.get("METRIQUE_LOGS")
+        logfile = os.path.join(logdir, logfile)
 
         logger_name = 'incon'
         logger = logging.getLogger(logger_name)
@@ -762,15 +803,17 @@ class Generic(pyclient):
         logger.propagate = 0
         self.log_inconsistency = logger.error
 
-    def sql_get_oids(self):
+    def sql_get_oids(self, where=None):
         '''
         Query source database for a distinct list of oids.
         '''
         table = self.get_property('table')
-        _id = self.get_property('column')
+        _oid = self.get_property('_oid')
         db = self.get_property('db')
-        sql = 'SELECT DISTINCT %s.%s FROM %s.%s' % (table, _id, db, table)
-        return sorted([r[0] for r in self.sql_proxy.fetchall(sql)])
+        sql = 'SELECT DISTINCT %s.%s FROM %s.%s' % (table, _oid, db, table)
+        if where:
+            sql += ' WHERE %s' % ' OR '.join(where)
+        return sorted([r[0] for r in self.fetchall(sql)])
 
     def _sql_distinct(self, sql):
         # whether to query for distinct rows only or not; default, no
@@ -779,12 +822,11 @@ class Generic(pyclient):
         else:
             return sql
 
-    def _sql_sort(self, table):
-        _id = self.get_property('column')
+    def _sql_sort(self, sql, table):
+        _oid = self.get_property('_oid')
         if self.get_property('sort', default=False):
-            return " ORDER BY %s.%s ASC" % (table, _id)
-        else:
-            return ""
+            sql += " ORDER BY %s.%s ASC" % (table, _oid)
+        return sql
 
     def _type_container(self, value, _type):
         ' apply type to all values in the list '
