@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
 # Author: "Juraj Niznan" <jniznan@redhat.com>
 
@@ -9,7 +10,9 @@ metrique.regtest
 This module regression test api related functionality
 '''
 
-raise NotImplemented("FIXME")
+raise NotImplementedError("FIXME")
+
+from __future__ import unicode_literals
 
 import logging
 import simplejson as json
